==> ttyrpld-1.24.tar.bz2: ttyrpld-1.24/user/global.c <==
/*=============================================================================
ttyrpld - TTY replay daemon
user/global.c - Shared functions for user-space apps
  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
  -- License restrictions apply (GPL v2)

  This file is part of ttyrpld.
  ttyrpld is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; however ONLY version 2 of the License.

  ttyrpld is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program kit; if not, write to:
  Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.

  -- For details see the COPYING file
=============================================================================*/
#include <sys/types.h>
#include <errno.h>
#include <sched.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "dev.h"
#include "global.h"

//-----------------------------------------------------------------------------
int count_args(const char **args) {
    int argk = 0;
    while(*args++ != NULL) { ++argk; }
    return argk;
}

ssize_t read_wait(int fd, void *buf, size_t count) {
    /* A wrapper for read() which guarantees that all bytes requested will
    be in BUF after read_wait() returns. (Except if there is an error.)
    Note that it will retry to read when it hits EOF, so only use this on files
    which are still being written to! */
    size_t rem = count;

    while(rem > 0) {
        ssize_t eax = read(fd, buf, rem);
        if(eax < 0) { return -errno; }
        if(eax == rem) { break; }
        buf += eax;
        rem -= eax;
        sched_yield(); // usleep(1) could also be a possibility
    }
    return count;
}

off_t G_skip(int fd, off_t offset, int do_wait) {
    /* For files (and stuff) that can not be seeked in, use a slurping method
    to get to the wanted position. This only works for forward offsets. */
    off_t seekable = lseek(fd, 0, SEEK_CUR);
    size_t rem = offset;
    char buf[4096];

    if(seekable != -1) { return lseek(fd, offset, SEEK_CUR); }

    if(do_wait) {
        while(rem > 0) {
            ssize_t eax = read(fd, buf, min_uint(sizeof(buf), rem));
            if(eax < 0) { return -1; }
            if(eax == rem) { return 0; }
            rem -= eax;
            sched_yield();
        }
        return 0;
    }

    // non wait method
    while(rem > 0) {
        ssize_t eax = read(fd, buf, min_uint(sizeof(buf), rem));
        if(eax <= 0) { return -1; }
        if(eax == rem) { return 0; }
        rem -= eax;
        sched_yield();
    }

    return 0;
}

void swab_be(void *srcp, size_t count) {
    char *movp = srcp - 1;
    unsigned char x;

    while(count) {
        x = *++movp;
        movp[0] = movp[1];
        *++movp = x;
        count -= 2;
    }

    return;
}

//==[ End of file ]============================================================

==> ttyrpld-1.42.tar.bz2: ttyrpld-1.42/user/global.c <==
/*=============================================================================
ttyrpld - TTY replay daemon
user/global.c - Shared functions for user-space apps
  Copyright © Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004 - 2005
  -- License restrictions apply (GPL v2)

  This file is part of ttyrpld.
  ttyrpld is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; however ONLY version 2 of the License.

  ttyrpld is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program kit; if not, write to:
  Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.

  -- For details see the COPYING file
=============================================================================*/
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include <sched.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <libHX.h>

#include "dev.h"
#include "global.h"

//-----------------------------------------------------------------------------
int count_args(const char **args) {
    int argk = 0;
    while(*args++ != NULL) { ++argk; }
    return argk;
}

void load_locale(const char *exe) {
    struct stat sb;
    int ok = 0;
    setlocale(LC_ALL, "");

    if(strchr(exe, '/') != NULL) {
        char *base = hmc_sinit(exe), *de;
        hmc_trunc(base, HX_strrcspn(base, "/"));
        hmc_strcat(&base, "/locale");
        de = hmc_dup(base);
        hmc_strcat(&de, "/de/LC_MESSAGES/ttyrpld.mo");
        if(stat(de, &sb) == 0) { bindtextdomain("ttyrpld", base); ++ok; }
        hmc_free(base);
        hmc_free(de);
    }
    if(!ok) {
        if(stat("locale/de/LC_MESSAGES/ttyrpld.mo", &sb) == 0) {
            bindtextdomain("ttyrpld", ".");
        } else if(stat("/usr/local/share/locale/de/LC_MESSAGES/ttyrpld.mo",
         &sb) == 0) {
            bindtextdomain("ttyrpld", "/usr/local/share/locale");
        } else if(stat("/usr/share/locale/de/LC_MESSAGES/ttyrpld.mo",
         &sb) == 0) {
            bindtextdomain("ttyrpld", "/usr/share/locale");
        }
    }

    textdomain("ttyrpld");
    return;
}

ssize_t read_wait(int fd, void *buf, size_t count) {
    /* A wrapper for read() which guarantees that all bytes requested will
    be in BUF after read_wait() returns. (Except if there is an error.)
    Note that it will retry to read when it hits EOF, so only use this on files
    which are still being written to! */
    size_t rem = count;

    while(rem > 0) {
        ssize_t eax = read(fd, buf, rem);
        if(eax < 0) { return -errno; }
        if(eax == rem) { break; }
        buf += eax;
        rem -= eax;
        sched_yield(); // usleep(1) could also be a possibility
    }
    return count;
}

off_t G_skip(int fd, off_t offset, int do_wait) {
    /* For files (and stuff) that can not be seeked in, use a slurping method
    to get to the wanted position. This only works for forward offsets. */
    off_t seekable = lseek(fd, 0, SEEK_CUR);
    size_t rem = offset;
    char buf[4096];

    if(seekable != -1) { return lseek(fd, offset, SEEK_CUR); }

    if(do_wait) {
        while(rem > 0) {
            ssize_t eax = read(fd, buf, min_uint(sizeof(buf), rem));
            if(eax < 0) { return -1; }
            if(eax == rem) { return 0; }
            rem -= eax;
            sched_yield();
        }
        return 0;
    }

    // non wait method
    while(rem > 0) {
        ssize_t eax = read(fd, buf, min_uint(sizeof(buf), rem));
        if(eax <= 0) { return -1; }
        if(eax == rem) { return 0; }
        rem -= eax;
        sched_yield();
    }

    return 0;
}

void swab_be(void *srcp, size_t count) {
    char *movp = srcp - 1;
    unsigned char x;

    while(count) {
        x = *++movp;
        movp[0] = movp[1];
        *++movp = x;
        count -= 2;
    }

    return;
}

//==[ End of file ]============================================================
